@@ -9,7 +9,7 @@
 *
 *	Contents:	analyse(), endobject()...: measurements on detections.
 *
-*	Last modify:	27/09/2005
+*	Last modify:	18/11/2008
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */
@@ -37,10 +37,13 @@
 #include	"image.h"
 #include	"photom.h"
 #include	"psf.h"
+#include	"profit.h"
 #include	"retina.h"
+#include	"som.h"
 #include	"winpos.h"
 
 static obj2struct	*obj2 = &outobj2;
+extern profitstruct	*theprofit;
 
 /********************************* analyse ***********************************/
 void  analyse(picstruct *field, picstruct *dfield, int objnb,
@@ -87,10 +90,10 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
    checkstruct		*check;
    pliststruct		*pixt;
    int			i,j,k,h, photoflag,area,errflag, cleanflag,
-			pospeakflag, profflag, minarea, gainflag;
+			pospeakflag, minarea, gainflag;
    double		tv,sigtv, ngamma,
 			esum, emx2,emy2,emxy, err,gain,backnoise2,dbacknoise2,
-			xm,ym, x,y,var,var2, profflux,proffluxvar;
+			xm,ym, x,y,var,var2;
    float		*heap,*heapt,*heapj,*heapk, swap;
    PIXTYPE		pix, cdpix, tpix, peak,cdpeak, thresh,dthresh;
    static PIXTYPE	threshs[NISO];
@@ -111,8 +114,7 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
     xm = ym = dbacknoise2 = 0.0;	/* to avoid gcc -Wall warnings */
 
   pospeakflag = FLAG(obj.peakx);
-  profflag = FLAG(obj.flux_prof);
-  gain = prefs.gain;
+  gain = field->gain;
   ngamma = field->ngamma;
   photoflag = (prefs.detect_type==PHOTO);
   gainflag = PLISTEXIST(var) && prefs.weightgain_flag;
@@ -142,7 +144,7 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
 
 
 /* Measure essential isophotal parameters in the measurement image... */
-  tv = sigtv = profflux = proffluxvar = 0.0;
+  tv = sigtv = 0.0;
   var = backnoise2 = field->backsig*field->backsig;
   peak = -BIG;
   cdpeak = -BIG;
@@ -177,12 +179,6 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
 
     sigtv += var2;
 
-    if (profflag && cdpix>0.0)
-      {
-      profflux += cdpix*pix;
-      proffluxvar += cdpix*var2;
-      }
-
     if (pix>thresh)
       area++;
     tv += pix;
@@ -244,12 +240,6 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
     free(heap);
     }
 
-  if (profflag)
-    {
-    obj->flux_prof = obj->fdflux>0.0? (float)(profflux/obj->fdflux) : 0.0;
-    obj->fluxerr_prof = obj->fdflux>0.0? (float)(proffluxvar/obj->fdflux):0.0;
-    }
-
   if (errflag)
     {
      double	flux2;
@@ -347,7 +337,7 @@ void  examineiso(picstruct *field, picstruct *dfield, objstruct *obj,
       mx = obj->mx;
       my = obj->my;
       dbkg = obj->dbkg;
-      sat = (double)(prefs.satur_level - obj->bkg);
+      sat = (double)(field->satur_level - obj->bkg);
       s = sx = sy = sxx = sxy = 0.0;
       for (pixt=pixel+obj->firstpix;pixt>=pixel;pixt=pixel+PLIST(pixt,nextpix))
         {
@@ -483,11 +473,6 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
 /*-- Convert the father of photom. error estimates from variance to RMS */
     obj2->flux_iso = obj->flux;
     obj2->fluxerr_iso = sqrt(obj->fluxerr);
-    if (FLAG(obj.flux_prof))
-      {
-      obj2->flux_prof = obj->flux_prof;
-      obj2->fluxerr_prof = sqrt(obj->fluxerr_prof);
-      }
 
     if (FLAG(obj2.flux_isocor))
       computeisocorflux(field, obj);
@@ -511,7 +496,7 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
       compute_winpos(field, wfield, obj);
 
 /*-- What about the peak of the profile? */
-    if (obj->peak+obj->bkg >= prefs.satur_level)
+    if (obj->peak+obj->bkg >= field->satur_level)
       obj->flag |= OBJ_SATUR;
 
 /*-- Check-image CHECK_APERTURES option */
@@ -540,7 +525,6 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
 
     if (FLAG(obj2.sprob))
       {
-       int	j;
        double	fac2, input[10], output, fwhm;
 
       fwhm = prefs.seeing_fwhm;
@@ -586,6 +570,29 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
       }
     obj->number = newnumber;
 
+/*-- SOM fitting */
+    if (prefs.somfit_flag)
+      {
+       float    *input;
+
+      input = thesom->input;
+      copyimage(field,input,thesom->inputsize[0],thesom->inputsize[1],ix,iy);
+
+      if (thesom->nextrainput)
+        {
+        input += thesom->ninput-thesom->nextrainput;
+        *(input) = (obj->mx+1)/field->width;
+        *(input+1) = (obj->my+1)/field->height;
+        }
+
+      som_phot(thesom, obj->bkg, field->backsig,
+        (float)field->gain, obj->mx-ix, obj->my-iy,
+        FLAG(obj2.vector_somfit)?outobj2.vector_somfit:NULL, -1.0);
+      obj2->stderr_somfit = thesom->stderror;
+      obj2->flux_somfit = thesom->amp;
+      outobj2.fluxerr_somfit = thesom->sigamp;
+      }
+
     if (FLAG(obj2.vignet))
       copyimage(field,outobj2.vignet,prefs.vignetsize[0],prefs.vignetsize[1],
 	ix,iy);
@@ -594,14 +601,14 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
       copyimage_center(field, outobj2.vigshift, prefs.vigshiftsize[0],
 		prefs.vigshiftsize[1], obj->mx, obj->my);
 
-/*--- Express everything in magnitude units */
-    computemags(field, obj);
-
 /*------------------------------- PSF fitting ------------------------------*/
     nsub = 1;
     if (prefs.psf_flag)
       {
-      psf_fit(thepsf, field, wfield, obj);
+      if (prefs.dpsf_flag)
+        double_psf_fit(ppsf, field, wfield, obj, thepsf, dfield, dwfield);
+      else
+        psf_fit(thepsf, field, wfield, obj);
       obj2->npsf = thepsfit->npsf;
       if (prefs.psfdisplay_type == PSFDISPLAY_SPLIT)
         {
@@ -618,12 +625,25 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
             obj2->y_psf[j] = thepsfit->y[j];
           if (FLAG(obj2.flux_psf) && j<prefs.psf_fluxsize)
             obj2->flux_psf[j] = thepsfit->flux[j];
+          if (FLAG(obj2.magerr_psf) && j<prefs.psf_magerrsize)
+            obj2->magerr_psf[j] = obj2->fluxerr_psf[j]>0.0?
+		1.086*obj2->fluxerr_psf[j]/thepsfit->flux[j] : 99.0;
+          if (FLAG(obj2.fluxerr_psf) && j<prefs.psf_fluxerrsize)
+            obj2->fluxerr_psf[j] = obj2->fluxerr_psf[j];     
           if (FLAG(obj2.mag_psf) && j<prefs.psf_magsize)
             obj2->mag_psf[j] = thepsfit->flux[j]>0.0?
 		prefs.mag_zeropoint -2.5*log10(thepsfit->flux[j]) : 99.0;
           }
       }
 
+/*----------------------------- Profile fitting -----------------------------*/
+    nsub = 1;
+    if (prefs.prof_flag)
+      profit_fit(theprofit, field, wfield, obj, obj2);
+
+/*--- Express everything in magnitude units */
+    computemags(field, obj);
+
 /*-------------------------------- Astrometry ------------------------------*/
     if (prefs.world_flag)
       computeastrom(field, obj);
@@ -643,10 +663,16 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
         if (FLAG(obj2.y_psf))
           obj2->y_psf[0] = thepsfit->y[j];
         if (FLAG(obj2.flux_psf))
-          obj2->flux_psf[0] = thepsfit->flux[j];
+          obj2->flux_psf[0] = thepsfit->flux[j]>0.0? thepsfit->flux[j]:0.0; /*?*/
         if (FLAG(obj2.mag_psf))
           obj2->mag_psf[0] = thepsfit->flux[j]>0.0?
 		prefs.mag_zeropoint -2.5*log10(thepsfit->flux[j]) : 99.0;
+        if (FLAG(obj2.magerr_psf))
+          obj2->magerr_psf[0]=
+		(thepsfit->flux[j]>0.0 && obj2->fluxerr_psf[j]>0.0) ? /*?*/
+			1.086*obj2->fluxerr_psf[j]/thepsfit->flux[j] : 99.0;
+        if (FLAG(obj2.fluxerr_psf))
+          obj2->fluxerr_psf[0]= obj2->fluxerr_psf[j];
         if (j)
           obj->number = ++thecat.ntotal;
         }
@@ -667,6 +693,25 @@ void	endobject(picstruct *field, picstruct *dfield, picstruct *wfield,
       writecat(n, objlist);
       }
     }
+  else
+    {
+/*-- Treatment of discarded detections */
+/*-- update segmentation map */
+    if ((check=prefs.check[CHECK_SEGMENTATION]))
+      {
+       ULONG	*pix;
+       ULONG	oldsnumber = obj->number;
+       int	dx,dx0,dy,dpix;
+
+      pix = (ULONG *)check->pix + check->width*obj->ymin + obj->xmin;
+      dx0 = obj->xmax-obj->xmin+1;
+      dpix = check->width-dx0;
+      for (dy=obj->ymax-obj->ymin+1; dy--; pix += dpix)
+        for (dx=dx0; dx--; pix++)
+          if (*pix==oldsnumber)
+            *pix = 0;
+      }
+    }
 
 /* Remove again from the image the object's pixels if BLANKing is on ... */
 /*-- ... and free memory */
