@@ -9,7 +9,7 @@
 *
 *	Contents:	functions for output of catalog data.
 *
-*	Last modify:	19/10/2005
+*	Last modify:	18/03/2009
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */
@@ -30,11 +30,13 @@
 #include	"sexhead.h"
 #include	"sexhead1.h"
 #include	"sexheadsc.h"
+#include	"xml.h"
 
 catstruct	*fitscat;
-tabstruct	*objtab;
+tabstruct	*objtab = NULL;
 FILE		*ascfile;
 char		*buf;
+int		catopen_flag = 0;
 
 /******************************* readcatparams *******************************/
 /*
@@ -101,6 +103,19 @@ void	readcatparams(char *filename)
   memset(&outobj2, 0, sizeof(outobj2));
   updateparamflags();
 
+  return;
+  }
+
+
+/******************************* alloccatparams ******************************/
+/*
+Allocate memory for parameter arrays
+*/
+void	alloccatparams(void)
+  {
+   keystruct	*key;
+   int		i;
+
 /* Go back to multi-dimensional arrays for memory allocation */
   if (thecat.nparam)
     for (i=objtab->nkey, key=objtab->key; i--; key = key->nextkey) 
@@ -118,6 +133,30 @@ void	readcatparams(char *filename)
   return;
   }
 
+/*************************** changecatparamarrays ****************************/
+/*
+Change parameter array dimensions
+*/
+void	changecatparamarrays(char *keyword, int *axisn, int naxis)
+  {
+   keystruct	*key;
+   int		d,i, size;
+
+  if (thecat.nparam)
+    for (i=objtab->nkey, key=objtab->key; i--; key = key->nextkey) 
+      if (key->naxis && !strcmp(keyword, key->name))
+        {
+        size = t_size[key->ttype];
+        if (key->naxis != naxis)
+          key->naxis = naxis;
+        for (d=0; d<naxis; d++)
+          size *= (key->naxisn[d]=axisn[d]);
+        key->nbytes = size;
+        break;
+        }
+
+  return;
+  }
 
 /***************************** updateparamflags ******************************/
 /*
@@ -128,6 +167,180 @@ void	updateparamflags()
   {
    int	i;
 
+/*----------------------------- Model-fitting -----------------------------*/
+
+  prefs.pattern_flag |= FLAG(obj2.prof_disk_patternvector)
+			| FLAG(obj2.prof_disk_patternmodvector)
+			| FLAG(obj2.prof_disk_patternargvector)
+			| FLAG(obj2.prof_disk_patternspiral);
+  FLAG(obj2.prof_disk_scale) |= prefs.pattern_flag;
+
+  FLAG(obj2.poserraw_prof) |= FLAG(obj2.poserrbw_prof);
+  FLAG(obj2.poserrcxxw_prof) |= FLAG(obj2.poserrcyyw_prof)
+			| FLAG(obj2.poserrcxyw_prof);
+  FLAG(obj2.poserrthetas_prof) |= FLAG(obj2.poserrtheta1950_prof)
+				| FLAG(obj2.poserrtheta2000_prof);
+  FLAG(obj2.poserrthetaw_prof) |= FLAG(obj2.poserrthetas_prof);
+
+  FLAG(obj2.poserrmx2w_prof) |= FLAG(obj2.poserrmy2w_prof)
+			| FLAG(obj2.poserrmxyw_prof)
+			| FLAG(obj2.poserrthetaw_prof)|FLAG(obj2.poserraw_prof)
+			| FLAG(obj2.poserrcxxw_prof);
+
+  FLAG(obj2.poserra_prof) |= FLAG(obj2.poserrb_prof)
+			| FLAG(obj2.poserrtheta_prof)
+			| FLAG(obj2.poserraw_prof);
+  FLAG(obj2.poserrcxx_prof) |= FLAG(obj2.poserrcyy_prof)
+			| FLAG(obj2.poserrcxy_prof);
+  FLAG(obj2.alpha1950_prof) |= FLAG(obj2.delta1950_prof)
+			| FLAG(obj2.poserrtheta1950_prof);
+  FLAG(obj2.alpha2000_prof) |= FLAG(obj2.delta2000_prof)
+			| FLAG(obj2.alpha1950_prof)
+			| FLAG(obj2.poserrtheta2000_prof);
+  FLAG(obj2.alphas_prof) |= FLAG(obj2.deltas_prof)
+			| FLAG(obj2.alpha2000_prof);
+  FLAG(obj2.xw_prof) |= FLAG(obj2.yw_prof)
+			| FLAG(obj2.alphas_prof);
+
+  FLAG(obj2.x_prof) |= FLAG(obj2.y_prof)
+			| FLAG(obj2.xw_prof)
+			| FLAG(obj2.poserra_prof)
+			| FLAG(obj2.poserrcxx_prof)
+			| FLAG(obj2.prof_concentration)
+			| FLAG(obj2.prof_class_star);
+  FLAG(obj2.mag_prof) |= FLAG(obj2.magerr_prof);
+  FLAG(obj2.flux_prof) |= FLAG(obj2.mag_prof) | FLAG(obj2.fluxerr_prof);
+  FLAG(obj2.prof_spheroid_mag) |= FLAG(obj2.prof_spheroid_magerr);
+  FLAG(obj2.prof_spheroid_reff) |= FLAG(obj2.prof_spheroid_refferr);
+  FLAG(obj2.prof_spheroid_aspect) |= FLAG(obj2.prof_spheroid_aspecterr);
+  FLAG(obj2.prof_spheroid_theta) |= FLAG(obj2.prof_spheroid_thetaerr);
+  FLAG(obj2.prof_spheroid_sersicn) |= FLAG(obj2.prof_spheroid_sersicnerr);
+  FLAG(obj2.prof_disk_mag) |= FLAG(obj2.prof_disk_magerr);
+  FLAG(obj2.prof_disk_scale) |= FLAG(obj2.prof_disk_scaleerr);
+  FLAG(obj2.prof_disk_aspect) |= FLAG(obj2.prof_disk_aspecterr);
+  FLAG(obj2.prof_disk_inclination) |= FLAG(obj2.prof_disk_inclinationerr);
+  FLAG(obj2.prof_disk_theta) |= FLAG(obj2.prof_disk_thetaerr);
+  FLAG(obj2.prof_bar_mag) |= FLAG(obj2.prof_bar_magerr);
+  FLAG(obj2.prof_bar_length) |= FLAG(obj2.prof_bar_lengtherr);
+  FLAG(obj2.prof_bar_aspect) |= FLAG(obj2.prof_bar_aspecterr);
+  FLAG(obj2.prof_bar_theta) |= FLAG(obj2.prof_bar_thetaerr);
+  FLAG(obj2.prof_arms_mag) |= FLAG(obj2.prof_arms_magerr);
+  FLAG(obj2.dtheta1950) |= FLAG(obj2.prof_spheroid_theta1950)
+			| FLAG(obj2.prof_disk_theta1950)
+//			| FLAG(obj2.prof_arms_theta1950)
+			| FLAG(obj2.prof_bar_theta1950)
+			| FLAG(obj2.poserrtheta1950_psf)
+			| FLAG(obj2.win_theta1950)
+			| FLAG(obj2.winposerr_theta1950)
+			| FLAG(obj2.poserr_theta1950)
+			| FLAG(obj2.theta1950)
+			| FLAG(obj2.poserrtheta1950_prof);
+  FLAG(obj2.dtheta2000) |= FLAG(obj2.prof_spheroid_theta2000)
+			| FLAG(obj2.prof_disk_theta2000)
+//			| FLAG(obj2.prof_arms_theta2000)
+			| FLAG(obj2.prof_bar_theta2000)
+			| FLAG(obj2.poserrtheta2000_psf)
+			| FLAG(obj2.win_theta2000)
+			| FLAG(obj2.winposerr_theta2000)
+			| FLAG(obj2.poserr_theta2000)
+			| FLAG(obj2.theta2000)
+			| FLAG(obj2.poserrtheta2000_prof);
+
+
+  FLAG(obj2.prof_spheroid_thetas) |= FLAG(obj2.prof_spheroid_theta2000)
+			| FLAG(obj2.prof_spheroid_theta1950);
+  FLAG(obj2.prof_spheroid_thetaw) |= FLAG(obj2.prof_spheroid_thetas);
+  FLAG(obj2.prof_disk_thetas) |= FLAG(obj2.prof_disk_theta2000)
+			| FLAG(obj2.prof_disk_theta1950);
+  FLAG(obj2.prof_disk_thetaw) |= FLAG(obj2.prof_disk_thetas);
+  FLAG(obj2.prof_bar_thetas) |= FLAG(obj2.prof_bar_theta2000)
+			| FLAG(obj2.prof_bar_theta1950);
+  FLAG(obj2.prof_bar_thetaw) |= FLAG(obj2.prof_bar_thetas);
+/*
+  FLAG(obj2.prof_arms_thetaw) |= FLAG(obj2.prof_arms_thetas)
+			| FLAG(obj2.prof_arms_theta2000)
+			| FLAG(obj2.prof_arms_theta1950);
+*/
+  FLAG(obj2.prof_arms_scalew) |= FLAG(obj2.prof_arms_startw)
+			| FLAG(obj2.prof_arms_scaleerrw)
+			| FLAG(obj2.prof_arms_starterrw);
+
+  FLAG(obj2.prof_bar_lengthw) |= FLAG(obj2.prof_bar_aspectw)
+			| FLAG(obj2.prof_bar_thetaw)
+			| FLAG(obj2.prof_bar_lengtherrw)
+			| FLAG(obj2.prof_bar_aspecterrw)
+			| FLAG(obj2.prof_bar_thetaerrw);
+
+  FLAG(obj2.prof_disk_scalew) |= FLAG(obj2.prof_disk_aspectw)
+			| FLAG(obj2.prof_disk_thetaw)
+			| FLAG(obj2.prof_disk_scaleerrw)
+			| FLAG(obj2.prof_disk_aspecterrw)
+			| FLAG(obj2.prof_disk_thetaerrw)
+			| FLAG(obj2.prof_arms_scalew);
+
+  FLAG(obj2.prof_spheroid_reffw) |= FLAG(obj2.prof_spheroid_aspectw)
+			| FLAG(obj2.prof_spheroid_thetaw)
+			| FLAG(obj2.prof_spheroid_refferrw)
+			| FLAG(obj2.prof_spheroid_aspecterrw)
+			| FLAG(obj2.prof_spheroid_thetaerrw);
+
+  FLAG(obj2.prof_flagw) |= FLAG(obj2.prof_spheroid_reffw)
+			| FLAG(obj2.prof_disk_scalew)
+			| FLAG(obj2.prof_bar_lengthw)
+			| FLAG(obj2.prof_arms_scalew);
+			
+  FLAG(obj2.prof_mx2) |= FLAG(obj2.prof_my2) | FLAG(obj2.prof_mxy)
+			| FLAG(obj2.prof_e1) |FLAG(obj2.prof_e2)
+			| FLAG(obj2.prof_eps1) |FLAG(obj2.prof_eps2);
+  FLAG(obj2.prof_arms_flux) |= FLAG(obj2.prof_arms_fluxerr)
+			| FLAG(obj2.prof_arms_mag)
+			| FLAG(obj2.prof_arms_scalew)
+			| FLAG(obj2.prof_arms_scale)
+			| FLAG(obj2.prof_arms_posang)
+			| FLAG(obj2.prof_arms_pitch)
+			| FLAG(obj2.prof_arms_start)
+			| FLAG(obj2.prof_arms_quadfrac);
+  FLAG(obj2.prof_bar_theta) |= FLAG(obj2.prof_bar_lengthw);
+  FLAG(obj2.prof_bar_flux) |= FLAG(obj2.prof_bar_fluxerr)
+			| FLAG(obj2.prof_bar_mag)
+			| FLAG(obj2.prof_bar_lengthw)
+			| FLAG(obj2.prof_bar_length)
+			| FLAG(obj2.prof_bar_aspect)
+			| FLAG(obj2.prof_bar_posang)
+			| FLAG(obj2.prof_bar_theta)
+			| FLAG(obj2.prof_arms_flux);
+  FLAG(obj2.prof_disk_flux) |= FLAG(obj2.prof_disk_fluxerr)
+			| FLAG(obj2.prof_disk_mag)
+			| FLAG(obj2.prof_disk_scalew)
+			| FLAG(obj2.prof_disk_scale)
+			| FLAG(obj2.prof_disk_aspect)
+			| FLAG(obj2.prof_disk_inclination)
+			| FLAG(obj2.prof_disk_theta)
+			| FLAG(obj2.prof_bar_flux);
+  FLAG(obj2.prof_spheroid_flux) |= FLAG(obj2.prof_spheroid_fluxerr)
+			| FLAG(obj2.prof_spheroid_mag)
+			| FLAG(obj2.prof_spheroid_reffw)
+			| FLAG(obj2.prof_spheroid_reff)
+			| FLAG(obj2.prof_spheroid_aspect)
+			| FLAG(obj2.prof_spheroid_theta)
+			| FLAG(obj2.prof_spheroid_sersicn);
+  prefs.prof_flag |= FLAG(obj2.prof_chi2) | FLAG(obj2.prof_niter)
+			| FLAG(obj2.prof_vector) | FLAG(obj2.prof_errvector)
+			| FLAG(obj2.x_prof) | FLAG(obj2.y_prof)
+			| FLAG(obj2.prof_mx2)
+			| FLAG(obj2.prof_disk_flux)
+			| FLAG(obj2.prof_spheroid_flux);
+
+
+/* If only global parameters are requested, fit a Sersic model */
+  if (prefs.prof_flag && !(FLAG(obj2.prof_spheroid_flux)
+			| FLAG(obj2.prof_disk_flux)))
+    {
+    FLAG(obj2.prof_spheroid_flux) |= prefs.prof_flag;
+    FLAG(obj2.prof_spheroid_sersicn) |= prefs.prof_flag;
+    }
+
+
 /*------------------------------ Astrometry ---------------------------------*/
   FLAG(obj2.win_aw) |= FLAG(obj2.win_bw) | FLAG(obj2.win_polarw);
   FLAG(obj2.win_cxxw) |= FLAG(obj2.win_cyyw) | FLAG(obj2.win_cxyw);
@@ -200,12 +413,12 @@ void	updateparamflags()
 
   FLAG(obj2.peakalpha1950) |= FLAG(obj2.peakdelta1950);
   FLAG(obj2.alpha1950) |= FLAG(obj2.delta1950) |  FLAG(obj2.theta1950)
-			| FLAG(obj2.poserr_theta1950);
+			| FLAG(obj2.poserr_theta1950) | FLAG(obj2.dtheta1950);
+;
   FLAG(obj2.peakalpha2000) |= FLAG(obj2.peakdelta2000)
 			| FLAG(obj2.peakalpha1950);
   FLAG(obj2.alpha2000) |= FLAG(obj2.delta2000) | FLAG(obj2.alpha1950)
-			| FLAG(obj2.theta2000)
-			| FLAG(obj2.poserr_theta2000);
+			| FLAG(obj2.dtheta2000);
   FLAG(obj2.peakalphas) |= FLAG(obj2.peakdeltas) | FLAG(obj2.peakalpha2000);
   FLAG(obj2.alphas) |= FLAG(obj2.deltas) | FLAG(obj2.alpha2000);
   FLAG(obj2.thetas) |= FLAG(obj2.theta1950) | FLAG(obj2.theta2000);
@@ -237,7 +450,7 @@ void	updateparamflags()
 
   FLAG(obj2.flux_best) |= FLAG(obj2.mag_best) | FLAG(obj2.fluxerr_best);
 
-  FLAG(obj2.hl_radius) |= FLAG(obj2.winpos_x);
+  FLAG(obj2.hl_radius) |= FLAG(obj2.winpos_x) | prefs.prof_flag;
 
   FLAG(obj2.flux_auto)  |= FLAG(obj2.mag_auto) | FLAG(obj2.magerr_auto)
 			| FLAG(obj2.fluxerr_auto)
@@ -258,9 +471,6 @@ void	updateparamflags()
   FLAG(obj2.flux_aper) |= FLAG(obj2.mag_aper)|FLAG(obj2.magerr_aper)
 			    | FLAG(obj2.fluxerr_aper);
 
-  FLAG(obj.flux_prof) |= FLAG(obj2.mag_prof)|FLAG(obj2.magerr_prof)
-			    | FLAG(obj2.flux_prof) | FLAG(obj2.fluxerr_prof);
-
   FLAG(obj2.flux_galfit) |= FLAG(obj2.mag_galfit) | FLAG(obj2.magerr_galfit)
 			    | FLAG(obj2.fluxerr_galfit);
 
@@ -328,6 +538,26 @@ void	updateparamflags()
   }
 
 
+/********************************** dumpparams *******************************/
+/*
+Initialize the catalog header
+*/
+void	dumpparams(void)
+  {
+   int		i;
+
+  for (i=0; *objkey[i].name ; i++)
+    if (*objkey[i].unit)
+      printf("#%-22.22s %-58.58s [%s]\n",
+		objkey[i].name, objkey[i].comment, objkey[i].unit);
+    else
+      printf("#%-22.22s %-58.58s\n",
+		objkey[i].name, objkey[i].comment);
+
+  return;
+  }
+
+
 /********************************** initcat **********************************/
 /*
 Initialize the catalog header
@@ -340,23 +570,23 @@ void	initcat(void)
   if (prefs.cat_type == CAT_NONE)
     return;
 
-  if (prefs.cat_type == ASCII_HEAD || prefs.cat_type == ASCII
-      || prefs.cat_type == ASCII_SKYCAT)
+  update_tab(objtab);
+  if (prefs.cat_type == ASCII_HEAD || prefs.cat_type == ASCII ||
+	prefs.cat_type == ASCII_SKYCAT || prefs.cat_type == ASCII_VO)
     {
     if (prefs.pipe_flag)
       ascfile = stdout;
     else
       if (!(ascfile = fopen(prefs.cat_name, "w+")))
         error(EXIT_FAILURE,"*Error*: cannot open ", prefs.cat_name);
-    update_tab(objtab);
     if (prefs.cat_type == ASCII_HEAD && (key = objtab->key))
       for (i=0,n=1; i++<objtab->nkey; key=key->nextkey)
         {
         if (*key->unit)
-          fprintf(ascfile, "# %3d %-15.15s %-47.47s [%s]\n",
+          fprintf(ascfile, "# %3d %-22.22s %-58.58s [%s]\n",
 		n, key->name,key->comment, key->unit);
         else
-          fprintf(ascfile, "# %3d %-15.15s %.47s\n",
+          fprintf(ascfile, "# %3d %-22.22s %-58.58s\n",
 		n, key->name,key->comment);
         n += key->nbytes/t_size[key->ttype];
         }
@@ -376,6 +606,12 @@ void	initcat(void)
         }
       fprintf(ascfile, "\n------------------\n");
       }
+    else if (prefs.cat_type == ASCII_VO && objtab->key) 
+      {
+      write_xml_header(ascfile);
+      write_vo_fields(ascfile);
+      fprintf(ascfile, "   <DATA><TABLEDATA>\n");
+      }
     }
   else
     {
@@ -405,6 +641,62 @@ void	initcat(void)
       }
     }
 
+  catopen_flag = 1;
+
+  return;
+  }
+
+
+/****** write_vo_fields *******************************************************
+PROTO	int	write_vo_fields(FILE *file)
+PURPOSE	Write the list of columns to an XML-VOTable file or stream
+INPUT	Pointer to the output file (or stream).
+OUTPUT	-.
+NOTES	-.
+AUTHOR	E. Bertin (IAP)
+VERSION	14/07/2006
+ ***/
+void	write_vo_fields(FILE *file)
+  {
+   keystruct	*key;
+   char		datatype[40], arraysize[40], str[40];
+   int		i, d;
+
+  if (!objtab || !objtab->key)
+    return;
+  key=objtab->key;
+  for (i=0; i++<objtab->nkey; key=key->nextkey)
+    {
+/*--- indicate datatype, arraysize, width and precision attributes */
+/*--- Handle multidimensional arrays */
+    arraysize[0] = '\0';
+    if (key->naxis>1)
+      {
+      for (d=0; d<key->naxis; d++)
+        {
+        sprintf(str, "%s%d", d?"x":" arraysize=\"", key->naxisn[d]);
+        strcat(arraysize, str);
+        }
+      strcat(arraysize, "\"");
+      }
+    switch(key->ttype)
+      {
+      case T_BYTE:	strcpy(datatype, "unsignedByte"); break;
+      case T_SHORT:	strcpy(datatype, "short"); break;
+      case T_LONG:	strcpy(datatype, "int"); break;
+      case T_FLOAT:	strcpy(datatype, "float"); break;
+      case T_DOUBLE:	strcpy(datatype, "double"); break;
+      default:		error(EXIT_FAILURE,
+			"*Internal Error*: Unknown datatype in ",
+			"initcat()");
+      }
+    fprintf(file,
+	"  <FIELD name=\"%s\" ucd=\"%s\" datatype=\"%s\" unit=\"%s\"%s>\n",
+	key->name, key->voucd, datatype,key->vounit, arraysize);
+    fprintf(file, "   <DESCRIPTION>%s</DESCRIPTION>\n", key->comment);
+    fprintf(file, "  </FIELD>\n");
+    }
+
   return;
   }
 
@@ -421,8 +713,8 @@ void	reinitcat(picstruct *field)
   if (prefs.cat_type == CAT_NONE)
     return;
 
-  if (prefs.cat_type != ASCII_HEAD && prefs.cat_type != ASCII
-      && prefs.cat_type != ASCII_SKYCAT)
+  if (prefs.cat_type != ASCII_HEAD && prefs.cat_type != ASCII &&
+	prefs.cat_type != ASCII_SKYCAT && prefs.cat_type != ASCII_VO)
     {
     update_tab(objtab);
     switch(prefs.cat_type)
@@ -430,9 +722,9 @@ void	reinitcat(picstruct *field)
       case FITS_LDAC:
 /*------ We create a dummy table (only used through its header) */
         QCALLOC(asctab, tabstruct, 1);
-        asctab->headnblock = field->fitsheadsize/FBSIZE;
-        QMALLOC(asctab->headbuf, char, asctab->headnblock*FBSIZE);
-        memcpy(asctab->headbuf, field->fitshead, asctab->headnblock*FBSIZE);
+        asctab->headnblock = field->tab->headnblock;
+        QMEMCPY(field->tab->headbuf, asctab->headbuf, char,
+		asctab->headnblock*FBSIZE);
         key = headkey;
         while (*key->name)
           addkeyto_head(asctab, key++);
@@ -458,9 +750,9 @@ void	reinitcat(picstruct *field)
       case FITS_TPX:
 /*------ We create a dummy table (only used through its header) */
         QCALLOC(asctab, tabstruct, 1);
-        asctab->headnblock = field->fitsheadsize/FBSIZE;
-        QMALLOC(asctab->headbuf, char, asctab->headnblock*FBSIZE);
-        memcpy(asctab->headbuf, field->fitshead, asctab->headnblock*FBSIZE);
+        asctab->headnblock = field->tab->headnblock;
+        QMEMCPY(field->tab->headbuf, asctab->headbuf, char,
+		asctab->headnblock*FBSIZE);
         key = headkey;
         while (*key->name)
           addkeyto_head(asctab, key++);
@@ -526,6 +818,9 @@ void	writecat(int n, objliststruct *objlist)
     case ASCII_SKYCAT:
       print_obj(ascfile, objtab);
       break;
+    case ASCII_VO:
+      voprint_obj(ascfile, objtab);
+      break;
 
     case CAT_NONE:
       break;
@@ -542,11 +837,17 @@ void	writecat(int n, objliststruct *objlist)
 /*
 Terminate the catalog output.
 */
-void	endcat()
+void	endcat(char *error)
   {
    keystruct	*key;
    int		i;
 
+  if (!catopen_flag)
+    {
+    if (prefs.cat_type == ASCII_VO)
+      write_xmlerror(prefs.cat_name, error);
+    return;
+    }
   switch(prefs.cat_type)
     {
     case ASCII:
@@ -556,9 +857,21 @@ void	endcat()
       break;
 
     case ASCII_SKYCAT:
+      fprintf(ascfile, skycattail);
+      if (!prefs.pipe_flag)
+        fclose(ascfile);
+      break;
+
+    case ASCII_VO:
+      fprintf(ascfile, "    </TABLEDATA></DATA>\n");
+      fprintf(ascfile, "  </TABLE>\n");
+/*---- Add configuration file meta-data */
+      write_xml_meta(ascfile, error);
+      fprintf(ascfile, "</RESOURCE>\n");
+      fprintf(ascfile, "</VOTABLE>\n");
+
       if (!prefs.pipe_flag)
         fclose(ascfile);
-      fprintf(ascfile, skycattail);
       break;
 
     case FITS_LDAC:
@@ -583,6 +896,7 @@ void	endcat()
   objtab->key = NULL;
   objtab->nkey = 0;
   free_tab(objtab);
+  objtab = NULL;
 
   return;
   }
@@ -604,6 +918,7 @@ void	reendcat()
     case ASCII:
     case ASCII_HEAD:
     case ASCII_SKYCAT:
+    case ASCII_VO:
       break;
 
     case FITS_LDAC:
@@ -616,8 +931,8 @@ void	reendcat()
       head = key->ptr;
       fitswrite(head, "SEXNDET ", &thecat.ndetect,H_INT,T_LONG);
       fitswrite(head, "SEXNFIN ", &thecat.ntotal, H_INT,T_LONG);
-      fitswrite(head, "SEXDATE ", thecat.ext_date, H_STRING, 0);
-      fitswrite(head, "SEXTIME ", thecat.ext_time, H_STRING, 0);
+      fitswrite(head, "SEXDATE ", thecat.ext_date, H_STRING, T_STRING);
+      fitswrite(head, "SEXTIME ", thecat.ext_time, H_STRING, T_STRING);
       fitswrite(head, "SEXELAPS", &thecat.ext_elapsed, H_FLOAT, T_DOUBLE);
       QFTELL(fitscat->file, pos, fitscat->filename);
       QFSEEK(fitscat->file, tab->headpos, SEEK_SET, fitscat->filename);
