@@ -9,7 +9,7 @@
 *
 *	Contents:	Compute magnitudes and other photometrical parameters.
 *
-*	Last modify:	24/08/2005
+*	Last modify:	02/05/2008
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */
@@ -61,7 +61,7 @@ void  computeaperflux(picstruct *field, picstruct *wfield,
   corrflag = (prefs.mask_type==MASK_CORRECT);
   gainflag = wfield && prefs.weightgain_flag;
   var = backnoise2 = field->backsig*field->backsig;
-  gain = prefs.gain;
+  gain = field->gain;
 /* Integration radius */
   raper = prefs.apert[i]/2.0;
   raper2 = raper*raper;
@@ -235,7 +235,7 @@ void  computepetroflux(picstruct *field, picstruct *dfield, picstruct *wfield,
   mx = obj->mx;
   my = obj->my;
   var = backnoise2 = field->backsig*field->backsig;
-  gain = prefs.gain;
+  gain = field->gain;
   pflag = (prefs.detect_type==PHOTO)? 1:0;
   corrflag = (prefs.mask_type==MASK_CORRECT);
   gainflag = wfield && prefs.weightgain_flag;
@@ -534,7 +534,7 @@ void  computeautoflux(picstruct *field, picstruct *dfield, picstruct *wfield,
   mx = obj->mx;
   my = obj->my;
   var = backnoise2 = field->backsig*field->backsig;
-  gain = prefs.gain;
+  gain = field->gain;
   pflag = (prefs.detect_type==PHOTO)? 1:0;
   corrflag = (prefs.mask_type==MASK_CORRECT);
   gainflag = wfield && prefs.weightgain_flag;
@@ -865,7 +865,7 @@ void  computemags(picstruct *field, objstruct *obj)
 			 1.086*obj2->fluxerr_somfit/obj2->flux_somfit
 			:99.0;
 
-/* Mag. PROFILE */
+/* Mag. models */
   if (FLAG(obj2.mag_prof))
     obj2->mag_prof = obj2->flux_prof>0.0?
 			 -2.5*log10(obj2->flux_prof) + prefs.mag_zeropoint
@@ -875,6 +875,50 @@ void  computemags(picstruct *field, objstruct *obj)
 			 1.086*obj2->fluxerr_prof/obj2->flux_prof
 			:99.0;
 
+  if (FLAG(obj2.prof_spheroid_mag))
+    obj2->prof_spheroid_mag = obj2->prof_spheroid_flux>0.0?
+			 -2.5*log10(obj2->prof_spheroid_flux)
+			+ prefs.mag_zeropoint
+			:99.0;
+  if (FLAG(obj2.prof_spheroid_magerr))
+    obj2->prof_spheroid_magerr = obj2->prof_spheroid_flux>0.0?
+			 1.086*obj2->prof_spheroid_fluxerr
+				/ obj2->prof_spheroid_flux
+			:99.0;
+
+  if (FLAG(obj2.prof_disk_mag))
+    obj2->prof_disk_mag = obj2->prof_disk_flux>0.0?
+			 -2.5*log10(obj2->prof_disk_flux)
+			+ prefs.mag_zeropoint
+			:99.0;
+  if (FLAG(obj2.prof_disk_magerr))
+    obj2->prof_disk_magerr = obj2->prof_disk_flux>0.0?
+			 1.086*obj2->prof_disk_fluxerr
+				/ obj2->prof_disk_flux
+			:99.0;
+
+  if (FLAG(obj2.prof_bar_mag))
+    obj2->prof_bar_mag = obj2->prof_bar_flux>0.0?
+			 -2.5*log10(obj2->prof_bar_flux)
+			+ prefs.mag_zeropoint
+			:99.0;
+  if (FLAG(obj2.prof_bar_magerr))
+    obj2->prof_bar_magerr = obj2->prof_bar_flux>0.0?
+			 1.086*obj2->prof_bar_fluxerr
+				/obj2->prof_bar_flux
+			:99.0;
+
+  if (FLAG(obj2.prof_arms_mag))
+    obj2->prof_arms_mag = obj2->prof_arms_flux>0.0?
+			 -2.5*log10(obj2->prof_arms_flux)
+			+ prefs.mag_zeropoint
+			:99.0;
+  if (FLAG(obj2.prof_arms_magerr))
+    obj2->prof_arms_magerr = obj2->prof_arms_flux>0.0?
+			 1.086*obj2->prof_arms_fluxerr
+				/obj2->prof_arms_flux
+			:99.0;
+
 /* Mag. WINdowed */
   if (FLAG(obj2.mag_win))
     obj2->mag_win = obj2->flux_win>0.0?
