@@ -5,11 +5,11 @@
 *
 *	Part of:	SExtractor
 *
-*	Author:		E.BERTIN, IAP/Leiden
+*	Author:		E.BERTIN, (IAP)
 *
 *	Contents:	functions dealing with the handling of pixel lists.
 *
-*	Last modify:	28/04/98
+*	Last modify:	29/11/2005
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */
@@ -36,9 +36,9 @@ typedef struct
 /*-------------------------------- globals ----------------------------------*/
 
 int	plistexist_value, plistexist_dvalue, plistexist_cdvalue,
-	plistexist_flag, plistexist_dthresh, plistexist_var,
-	plistoff_value, plistoff_dvalue,plistoff_cdvalue,
-	plistoff_flag[MAXFLAG], plistoff_dthresh, plistoff_var,
+	plistexist_flag, plistexist_wflag, plistexist_dthresh, plistexist_var,
+	plistoff_value, plistoff_dvalue, plistoff_cdvalue,
+	plistoff_flag[MAXFLAG], plistoff_wflag, plistoff_dthresh, plistoff_var,
 	plistsize;
 
 /*------------------------------- functions ---------------------------------*/
